/* ESP32_sMQTTBroker
 *  
 * sMQTTBroker demo for ESP32 (C++-style)
 * 
 * A program egyedi MQTT brókert definiál, a virtuális visszakívási 
 * függvények felüldefiniálásával.
 * Az onConnect függvényt jelszóellenőrzéssel egészítettük ki.
 * Mindegyik visszahívási függvényt naplózással egészítettük ki.
 * Kapcsolódjunk egy külső klienssel, és publikáljunk valamit!
 * A soros porthoz kapcsolódó terminál ablakban követhetjük az eseményeket.
 * 
 * Forrás: https://github.com/terrorsl/sMQTTBroker
 * A felhasznált mintapélda: examples/advanvebroker.ino
 * Szerző: Vyacheslav Shiryaev
 * Kisebb módosítások: I. Cserny, Március 9., 2022. 
 * 
 * Tested with Arduino IDE 1.8.19 and ESP32 Arduino core 2.0.2
 */

#include <WiFi.h>
#include <sMQTTBroker.h>
#include "secrets.h"
#include <iostream>
#include <string>
#include <stdio.h>

class MyBroker: public sMQTTBroker {
  public:
    bool onConnect(sMQTTClient *client, const std::string &username, const std::string &password)
    { if( (0 == username.compare(uMQTT_USER)) && (0 == password.compare(uMQTT_PASS))) {
      /* if ((0 == strcmp(username.c_str(), uMQTT_USER)) && (0 == strcmp(password.c_str(), uMQTT_PASS))) { */
        std::printf("Client %s connected\r\n", client->getClientId().c_str());
        return true;
      }
      else {
        std::printf("Client %s failed to connect\r\n", client->getClientId().c_str());
        return false;
      }
    };
    void onRemove(sMQTTClient* client)
    {
      std::printf("Client %s disconnected\r\n", client->getClientId().c_str());

    };
    void onPublish(sMQTTClient *client, const std::string &topic, const std::string &payload)
    {
      // client publish a message to the topic
      // you can parse the message, save it to a database, or whatever, based on your goals
      std::printf("Client %s published %s into %s\r\n", client->getClientId().c_str(), payload.c_str(), topic.c_str());
    }
    bool onEvent(sMQTTEvent *event)
    {
      switch (event->Type())
      {
        case NewClient_sMQTTEventType:
          {
            sMQTTNewClientEvent *e = (sMQTTNewClientEvent*)event;
            e->Login();
            e->Password();
          }
          break;
        case LostConnect_sMQTTEventType:
          WiFi.reconnect();
          break;
      }
      return true;
    }
};

MyBroker broker;

void setup_wifi() {
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(WIFI_SSID);
  WiFi.mode(WIFI_STA);
  WiFi.begin(WIFI_SSID, WIFI_PASS);
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("WiFi connected");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());
}

void setup() {
  Serial.begin(115200);
  setup_wifi();
  broker.init(1883);
};

void loop() {
  broker.update();
};
